<?php
/**
 * Static class that will handle all form submission from customer
 *
 * @author MoreConvert
 * @package Smart Wishlist For More Convert
 * @version 1.3.3
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'WLFMC_Form_Handler' ) ) {
	/**
	 * WooCommerce Smart Wishlist Form Handler
	 */
	class WLFMC_Form_Handler {
		/**
		 * Performs all required add_actions to handle forms
		 *
		 * @return void
		 */
		public static function init() {
			/**
			 * This check was added to prevent bots from accidentally executing wishlist code
			 */
			if ( ! self::process_form_handling() ) {
				return;
			}

			// add to wishlist when js is disabled.
			add_action( 'init', array( 'WLFMC_Form_Handler', 'add_to_wishlist' ) );

			// remove from wishlist when js is disabled.
			add_action( 'init', array( 'WLFMC_Form_Handler', 'remove_from_wishlist' ) );

			// remove from wishlist after add to cart.
			add_action( 'woocommerce_add_to_cart', array( 'WLFMC_Form_Handler', 'remove_from_wishlist_after_add_to_cart' ), 10, 6 );

			// these actions manage cart, and needs to hooked to wp_loaded.
			add_action( 'wp_loaded', array( 'WLFMC_Form_Handler', 'apply_bulk_actions' ), 15 );
			add_action( 'wp_loaded', array( 'WLFMC_Form_Handler', 'add_all_to_cart' ), 15 );
			add_action( 'wp_loaded', array( 'WLFMC_Form_Handler', 'add_to_cart' ), 15 );

			add_action( 'init', array( 'WLFMC_Form_Handler', 'unsubscribe' ) );

		}

		/**
		 * Return true if system can process request; false otherwise
		 *
		 * @return bool
		 */
		public static function process_form_handling(): bool {
			$user_agent = isset( $_SERVER['HTTP_USER_AGENT'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_USER_AGENT'] ) ) : false;

			if ( $user_agent && apply_filters( 'wlfmc_block_user_agent', preg_match( '/bot|crawl|slurp|spider|wordpress/i', $user_agent ), $user_agent ) ) {
				return false;
			}

			return true;
		}

		/**
		 * Apply bulk actions to wishlist items
		 *
		 * @return void
		 * @version 1.3.2
		 */
		public static function apply_bulk_actions() {

			if ( ! isset( $_POST['wlfmc_edit_wishlist'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['wlfmc_edit_wishlist'] ) ), 'wlfmc_edit_wishlist_action' ) || ( ! isset( $_POST['add_selected_to_cart'] ) && ! isset( $_POST['apply_bulk_actions'] ) ) || empty( $_POST['items'] ) ) {
				return;
			}

			$wishlist_id = isset( $_POST['wishlist_id'] ) ? sanitize_text_field( wp_unslash( $_POST['wishlist_id'] ) ) : false;
			$action      = isset( $_POST['bulk_actions'] ) ? sanitize_text_field( wp_unslash( $_POST['bulk_actions'] ) ) : false;
			$items       = wp_unslash( $_POST['items'] ) ; // phpcs:ignore

			if ( isset( $_POST['add_selected_to_cart'] ) ) {
				$action = 'add_to_cart';
			}

			if ( ! $wishlist_id || ! $action ) {
				return;
			}

			if ( empty( $items ) ) {
				wc_add_notice( esc_html__( 'You have to select at least one product', 'wc-wlfmc-wishlist' ), 'error' );
			}

			$wishlist = wlfmc_get_wishlist( $wishlist_id );

			if ( ! $wishlist ) {
				return;
			}

			$options                  = new MCT_Options( 'wlfmc_options' );
			$remove_after_add_to_cart = 'added-to-cart' === $options->get_option( 'remove_from_wishlist', 'none' );
			$redirect_to_cart         = $options->get_option( 'redirect_after_add_to_cart', true );

			$processed = array();
			$errors    = array();

			foreach ( $items as $item_id => $prop ) {
				if ( empty( $prop['cb'] ) ) {
					continue;
				}
				$result = false;
				$item   = $wishlist->get_product( $item_id );

				if ( ! $item ) {
					continue;
				}

				switch ( $action ) {
					case 'add_to_cart':
						$product_id = $item->get_product_id();

						try {
							$product = $item->get_product();
							if ( $product && $product->is_type( 'variable' ) ) {
								wc_add_notice( apply_filters( 'wlfmc_add_all_to_cart_error_message_for_variable', __( 'you don\'t select a variation for it', 'wc-wlfmc-wishlist' ), $product ), 'error' );
								$errors[ $product_id ] = self::merge_notices( wp_strip_all_tags( $item->get_formatted_product_name() ) );
								continue 2;
							}

							$quantity   = $item->get_quantity();
							$meta       = $item->get_product_meta( 'view' );
							$cart_item  = $item->get_cart_item();
							$attributes = array();
							if ( isset( $meta['attributes'] ) && ! empty( $meta['attributes'] ) ) {
								foreach ( $meta['attributes'] as $key => $value ) {
									if ( '' !== $value ) {
										$attributes[ $key ] = $value;
									}
								}
							}
							$variation_id = 0;
							if ( $product && 'variation' === $product->get_type() ) {
								$variation_id = $product_id;
								$product_id   = $product->get_parent_id();
							}
							$passed_validation = apply_filters( 'wlfmc_woocommerce_add_to_cart_validation', true, $product, $meta, $item, $cart_item );

							if ( $passed_validation ) {
								$result = WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $attributes, $cart_item );
							}

							if ( false !== $result && $remove_after_add_to_cart ) {
								$item->delete();

								do_action( 'wlfmc_product_added_to_cart', $wishlist->get_id(), $result, $product_id, $quantity );
							}
							if ( ! $result ) {
								$errors[ $product_id ] = self::merge_notices( wp_strip_all_tags( $item->get_formatted_product_name() ) );
							}
						} catch ( Exception $e ) {
							$errors[ $product_id ] = self::merge_notices( wp_strip_all_tags( $item->get_formatted_product_name() ) );
							continue 2;
						}
						break;

					case 'delete':
						$result = $item->delete();
						break;
					default:
						// maybe customer wants to move items to another list.
						$destination_wishlist = wlfmc_get_wishlist( $action );

						if ( ! $destination_wishlist ) {
							continue 2;
						}

						$item->set_wishlist_id( $destination_wishlist->get_id() );
						$item->set_date_added( current_time( 'mysql' ) );
						$result = $item->save();
				}

				if ( $result ) {
					$processed[] = $item;
				}
			}

			if ( ! empty( $processed ) ) {
				$count = count( $processed );
				switch ( $action ) {
					case 'add_to_cart':
						/* translators: %s: Number of successful operations . */
						$message = esc_html( sprintf( _n( '%s item have been correctly added to the cart', '%s items have been correctly added to the cart', $count, 'wc-wlfmc-wishlist' ), $count ) );
						break;
					case 'delete':
						/* translators: %s: Number of successful operations . */
						$message = esc_html( sprintf( _n( '%s item have been correctly removed', '%s items have been correctly removed', $count, 'wc-wlfmc-wishlist' ), $count ) );

						break;
					default:
						// translators: 1. Destination wishlist name.
						$message = sprintf( __( 'The items have been correctly moved to %s', 'wc-wlfmc-wishlist' ), $destination_wishlist->get_formatted_name() );
				}

				wc_add_notice( $message );
			} elseif ( 'add_to_cart' !== $action ) {
				wc_add_notice( esc_html__( 'An error occurred while processing this action', 'wc-wlfmc-wishlist' ), 'error' );
			}
			if ( ! empty( $errors ) && 'add_to_cart' === $action ) {
				wc_add_notice( implode( '<br>', $errors ), 'error' );
			}

			$cart_url     = wc_get_cart_url();
			$redirect_url = isset( $_REQUEST['redirect_to'] ) ? esc_url_raw( wp_unslash( $_REQUEST['redirect_to'] ) ) : $wishlist->get_url();
			$redirect_url = ( 'add_to_cart' === $action && $redirect_to_cart && ! empty( $processed ) ) ? $cart_url : $redirect_url;

			wp_safe_redirect( $redirect_url );
			die();
		}

		/**
		 * Add all items of a wishlist to cart
		 *
		 * @return void
		 * @version 1.3.0
		 */
		public static function add_all_to_cart() {

			if ( ! isset( $_POST['wlfmc_edit_wishlist'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['wlfmc_edit_wishlist'] ) ), 'wlfmc_edit_wishlist_action' ) || ! isset( $_REQUEST['add_all_to_cart'] ) ) {
				return;
			}

			$wishlist_id = isset( $_REQUEST['wishlist_id'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['wishlist_id'] ) ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			$wishlists   = array();

			if ( $wishlist_id ) {
				$wishlist = wlfmc_get_wishlist( $wishlist_id );

				if ( ! $wishlist ) {
					return;
				}

				$wishlists[] = $wishlist;
			} else {
				$wishlists = WLFMC()->get_current_user_wishlists();
			}

			$options                  = new MCT_Options( 'wlfmc_options' );
			$remove_after_add_to_cart = 'added-to-cart' === $options->get_option( 'remove_from_wishlist', 'none' );
			$redirect_to_cart         = $options->get_option( 'redirect_after_add_to_cart', true );

			$processed = array();
			$errors    = array();
			remove_action(
				'woocommerce_add_to_cart',
				array(
					'WLFMC_Form_Handler',
					'remove_from_wishlist_after_add_to_cart',
				)
			);

			do_action( 'wlfmc_before_add_all_to_cart_from_wishlist', $wishlists );

			if ( apply_filters( 'wlfmc_add_all_to_cart_from_wishlist', ! empty( $wishlists ) ) ) {
				foreach ( $wishlists as $wishlist ) {
					if ( $wishlist->has_items() ) {
						foreach ( $wishlist->get_items() as $item ) {

							$product    = $item->get_product();
							$product_id = $item->get_product_id();
							if ( $product && $product->is_type( 'variable' ) ) {
								wc_add_notice( apply_filters( 'wlfmc_add_all_to_cart_error_message_for_variable', __( 'you don\'t select a variation for it', 'wc-wlfmc-wishlist' ), $product ), 'error' );
								$errors[ $product_id ] = self::merge_notices( wp_strip_all_tags( $item->get_formatted_product_name() ) );
								continue;
							}

							try {
								$quantity   = $item->get_quantity();
								$meta       = $item->get_product_meta( 'view' );
								$cart_item  = $item->get_cart_item();
								$attributes = array();
								if ( isset( $meta['attributes'] ) && ! empty( $meta['attributes'] ) ) {
									foreach ( $meta['attributes'] as $key => $value ) {
										if ( '' !== $value ) {
											$attributes[ $key ] = $value;
										}
									}
								}
								$variation_id = 0;
								if ( $product && 'variation' === $product->get_type() ) {
									$variation_id = $product_id;
									$product_id   = $product->get_parent_id();
								}
								$passed_validation = apply_filters( 'wlfmc_woocommerce_add_to_cart_validation', true, $product, $meta, $item, $cart_item );
								$cart_item_key     = false;
								if ( $passed_validation ) {
									$cart_item_key = WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $attributes, $cart_item );
								}
								if ( false !== $cart_item_key ) {
									$processed[] = $item;

									if ( $remove_after_add_to_cart ) {
										$item->delete();

										do_action( 'wlfmc_product_added_to_cart', $wishlist->get_id(), $cart_item_key, $product_id, $quantity );
									}
								} else {
									$errors[ $product_id ] = self::merge_notices( wp_strip_all_tags( $item->get_formatted_product_name() ) );
								}
							} catch ( Exception $e ) {
								$errors[ $product_id ] = self::merge_notices( wp_strip_all_tags( $item->get_formatted_product_name() ) );
								continue;
							}
						}
					}
				}
			}

			if ( ! empty( $processed ) ) {
				/* translators: %s: Number of successful operations . */
				$message = esc_html( sprintf( _n( '%s item have been correctly added to the cart', '%s items have been correctly added to the cart', count( $processed ), 'wc-wlfmc-wishlist' ), count( $processed ) ) );

				wc_add_notice( $message );
			}
			if ( ! empty( $errors ) ) {
				wc_add_notice( implode( '<br>', $errors ), 'error' );
			}

			$cart_url     = wc_get_cart_url();
			$redirect_url = $wishlist_id ? $wishlist->get_url() : remove_query_arg( array( 'add_all_to_cart' ) );
			$redirect_url = isset( $_REQUEST['redirect_to'] ) ? esc_url_raw( wp_unslash( $_REQUEST['redirect_to'] ) ) : $redirect_url; // phpcs:ignore WordPress.Security.NonceVerification
			$redirect_url = $redirect_to_cart && ! empty( $processed ) ? $cart_url : $redirect_url;

			wp_safe_redirect( $redirect_url );
			die;
		}

		/**
		 * Merge all validation error notice
		 *
		 * @param string $product_name Formated product name.
		 *
		 * @return string|void
		 */
		public static function merge_notices( $product_name ) {
			if ( 0 === wc_notice_count( 'error' ) ) {
				wc_clear_notices();
				return;
			}

			$error_notices = wc_get_notices( 'error' );
			$error_notices = array_map(
				function( $val ) {
					return $val['notice'];
				},
				$error_notices
			);

			// Prevent notices from being output later on.
			wc_clear_notices();
			/* translators: %1$s: product name , %2$: error notice . */
			return esc_html( sprintf( __( 'We couldn\'t add %1$s to the cart because: %2$s', 'wc-wlfmc-wishlist' ), $product_name, implode( ',', $error_notices ) ) );

		}
		/**
		 * Add to cart item
		 *
		 * @return void
		 * @throws Exception If add to cart fails.
		 * @since 1.4.2
		 */
		public static function add_to_cart() {
			if ( ! isset( $_REQUEST['action'] ) || ! isset( $_REQUEST['_wpnonce'] ) || 'wlfmc_add_to_cart' !== $_REQUEST['action'] || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'wlfmc_add_to_cart' ) ) {
				return;
			}

			$item_id     = isset( $_REQUEST['lid'] ) ? absint( $_REQUEST['lid'] ) : '';
			$wishlist_id = isset( $_REQUEST['wid'] ) ? absint( $_REQUEST['wid'] ) : '';

			if ( ! $item_id || ! $wishlist_id ) {
				return;
			}

			$wishlist = wlfmc_get_wishlist( $wishlist_id );

			if ( ! $wishlist ) {
				return;
			}

			$options                  = new MCT_Options( 'wlfmc_options' );
			$remove_after_add_to_cart = 'added-to-cart' === $options->get_option( 'remove_from_wishlist', 'none' );
			$redirect_to_cart         = $options->get_option( 'redirect_after_add_to_cart', true );

			$item = $wishlist->get_item( $item_id );

			if ( ! $item ) {
				return;
			}

			$product_id = $item->get_product_id();
			$quantity   = $item->get_quantity();
			$meta       = $item->get_product_meta( 'view' );
			$cart_item  = $item->get_cart_item();
			$product    = $item->get_product();
			$attributes = array();
			if ( isset( $meta['attributes'] ) && ! empty( $meta['attributes'] ) ) {
				foreach ( $meta['attributes'] as $key => $value ) {
					if ( '' !== $value ) {
						$attributes[ $key ] = $value;
					}
				}
			}
			$variation_id = 0;
			if ( $product && 'variation' === $product->get_type() ) {
				$variation_id = $product_id;
				$product_id   = $product->get_parent_id();
			}
			$passed_validation = apply_filters( 'wlfmc_woocommerce_add_to_cart_validation', true, $product, $meta, $item, $cart_item );
			if ( $passed_validation ) {
				$cart_item_key = WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $attributes, $cart_item );

				if ( false !== $cart_item_key ) {
					wc_add_to_cart_message( array( $product_id => $quantity ), true );
					if ( $remove_after_add_to_cart ) {
						$item->delete();
						do_action( 'wlfmc_product_added_to_cart', $wishlist->get_id(), $cart_item_key, $product_id, $quantity );
					}
				}
				$cart_url     = wc_get_cart_url();
				$redirect_url = isset( $_REQUEST['redirect_to'] ) ? esc_url_raw( wp_unslash( $_REQUEST['redirect_to'] ) ) : $wishlist->get_url();
				$redirect_url = ( true === $cart_item_key && $redirect_to_cart ) ? $cart_url : $redirect_url;

			} else {
				$product      = $item->get_product();
				$permalink    = apply_filters( 'woocommerce_cart_item_permalink', $product->is_visible() ? $product->get_permalink( $cart_item ) : '', $cart_item, '' );
				$redirect_url = apply_filters( 'woocommerce_cart_redirect_after_error', $permalink, $product_id );
			}

			wp_safe_redirect( $redirect_url );
			die();
		}

		/**
		 * Adds a product to wishlist when js is disabled
		 *
		 * @return void
		 *
		 * @version 1.3.1
		 */
		public static function add_to_wishlist() {
			// add item to wishlist when javascript is not enabled.
			if ( isset( $_GET['add_to_wishlist'] ) ) {// phpcs:ignore WordPress.Security.NonceVerification
				try {
					$result = WLFMC()->add();

					$options                        = new MCT_Options( 'wlfmc_options' );
					$click_wishlist_button_behavior = $options->get_option( 'click_wishlist_button_behavior', 'just-add' );

					do_action( 'wlfmc_analytics_added_to_wishlist', intval( $_GET['add_to_wishlist'] ), $result['wishlist_id'], $result['user_id'] );// phpcs:ignore WordPress.Security.NonceVerification

					if ( '' !== $options->get_option( 'product_added_text' ) ) {
						wc_add_notice( $options->get_option( 'product_added_text' ) );

					}

					if ( 'add-redirect' === $click_wishlist_button_behavior ) {
						$wishlist_url = is_user_logged_in() ? apply_filters( 'wlfmc_user_logged_in_wishlist_url', wc_get_account_endpoint_url( get_option( 'woocommerce_myaccount_wlfmc_wishlist_endpoint', 'wlfmc-wishlist' ) ) ) : WLFMC()->get_last_operation_url();
						wp_safe_redirect( $wishlist_url );
						die();
					}
				} catch ( Exception $e ) {
					wc_add_notice( apply_filters( 'wlfmc_error_adding_to_wishlist_message', $e->getMessage() ), 'error' );
				}
			}
		}

		/**
		 * Removes from wishlist when js is disabled
		 *
		 * @return void
		 */
		public static function remove_from_wishlist() {
			// remove item from wishlist when javascript is not enabled.
			if ( isset( $_GET['remove_from_wishlist'] ) ) {// phpcs:ignore WordPress.Security.NonceVerification
				try {
					WLFMC()->remove();
					$options = new MCT_Options( 'wlfmc_options' );
					if ( '' !== $options->get_option( 'product_removed_text' ) ) {
						wc_add_notice( $options->get_option( 'product_removed_text' ) );
					}
				} catch ( Exception $e ) {
					wc_add_notice( $e->getMessage(), 'error' );
				}
			}
		}

		/**
		 * Remove from wishlist after adding to cart
		 *
		 * @param string     $cart_item_key cart product unique key.
		 * @param int        $product_id Product id.
		 * @param int        $quantity Quantity.
		 * @param int        $variation_id Variation id.
		 * @param null|array $variation attribute values.
		 * @param array      $cart_item_data extra cart item data we want to pass into the item.
		 *
		 * @return void
		 */
		public static function remove_from_wishlist_after_add_to_cart( string $cart_item_key, int $product_id, int $quantity, int $variation_id, $variation, array $cart_item_data ) {
			$options = new MCT_Options( 'wlfmc_options' );

			if ( 'added-to-cart' !== $options->get_option( 'remove_from_wishlist', 'none' ) ) {
				return;
			}

			$args = array();
			// phpcs:disable WordPress.Security.NonceVerification
			if ( isset( $_REQUEST['remove_from_wishlist_after_add_to_cart'] ) ) {

				$args['remove_from_wishlist'] = intval( $_REQUEST['remove_from_wishlist_after_add_to_cart'] );

				if ( isset( $_REQUEST['wishlist_id'] ) ) {
					$args['wishlist_id'] = sanitize_text_field( wp_unslash( $_REQUEST['wishlist_id'] ) );
				}
			} elseif ( wlfmc_is_wishlist() && isset( $_REQUEST['add-to-cart'] ) ) {
				$args['remove_from_wishlist'] = intval( $_REQUEST['add-to-cart'] );

				if ( isset( $_REQUEST['wishlist_id'] ) ) {
					$args['wishlist_id'] = sanitize_text_field( wp_unslash( $_REQUEST['wishlist_id'] ) );
				}
			}
			// phpcs:enable WordPress.Security.NonceVerification
			if ( ! empty( $args['wishlist_id'] ) ) {

				try {

					$result = WLFMC()->remove( $args );

					do_action( 'wlfmc_product_added_to_cart', $result['wishlist_id'], $cart_item_key, $args['remove_from_wishlist'], $quantity );

				} catch ( Exception $e ) {
					// we were unable to remove item from the wishlist; no follow up is provided.
					wc_add_notice( $e->getMessage(), 'error' );
				}
			}
		}

		/**
		 * Unsubscribe from mailing lists
		 *
		 * @return void
		 * @since 1.4.0
		 */
		public static function unsubscribe() {
			global $wpdb;
			// phpcs:disable WordPress.Security.NonceVerification
			if ( ! isset( $_GET['wlfmc_unsubscribe'] ) ) {
				return;
			}

			// retrieve unsubscription_process.
			$unsubscribe_token = sanitize_text_field( wp_unslash( $_GET['wlfmc_unsubscribe'] ) );

			if ( ! $unsubscribe_token ) {
				return;
			}

			// if user is not logged in, send to login page.
			if ( ! is_user_logged_in() ) {
				wc_add_notice( esc_html__( 'You must be logged in to unsubscribe', 'wc-wlfmc-wishlist' ), 'notice' );
				wp_safe_redirect( add_query_arg( 'redirect', esc_url( add_query_arg( $_GET, get_home_url() ) ), wc_get_page_permalink( 'myaccount' ) ) );
				die;
			}
			// phpcs:enable WordPress.Security.NonceVerification

			// redirect uri.
			$redirect = apply_filters( 'wlfmc_after_unsubscribe_redirect', get_permalink( wc_get_page_id( 'shop' ) ) );

			// get current user token.
			$user_id                      = get_current_user_id();
			$user                         = wp_get_current_user();
			$user_unsubscribe_token       = get_user_meta( $user_id, 'wlfmc_unsubscribe_token', true );
			$unsubscribe_token_expiration = get_user_meta( $user_id, 'wlfmc_unsubscribe_token_expiration', true );

			// check for match with provided token.
			if ( $unsubscribe_token !== $user_unsubscribe_token ) {
				wc_add_notice( esc_html__( 'The token provided does not match the current user', 'wc-wlfmc-wishlist' ), 'notice' );
				wp_safe_redirect( $redirect );
				die;
			}

			if ( $unsubscribe_token_expiration < time() ) {
				wc_add_notice( esc_html__( 'The token provided is expired; contact us to so we can manually unsubscribe your from the list', 'wc-wlfmc-wishlist' ), 'notice' );
				wp_safe_redirect( $redirect );
				die;
			}

			$unsubscribed_users = get_option( 'wlfmc_unsubscribed_users', array() );

			if ( ! in_array( $user->user_email, $unsubscribed_users, true ) ) {
				$unsubscribed_users[] = $user->user_email;

				update_option( 'wlfmc_unsubscribed_users', $unsubscribed_users );

				$wpdb->query( $wpdb->prepare( "UPDATE $wpdb->wlfmc_offers SET status = 'unsubscribed' WHERE user_id = %d AND status IN ('sending' ,'not-send') ", $user->ID ) );// db call ok; no-cache ok.

				do_action( 'wlfmc_unsubscribed_user', $user, $unsubscribed_users );

			}

			wc_add_notice( defined( 'MC_WLFMC_PREMIUM' ) ? esc_html__( 'You have unsubscribed and you will not receive any more emails and offers unless you enable the settings from your user panel.', 'wc-wlfmc-wishlist' ) : esc_html__( 'You have unsubscribed and you will not receive any more emails and offers.', 'wc-wlfmc-wishlist' ) );
			wp_safe_redirect( $redirect );
			die;
		}

	}
}

WLFMC_Form_Handler::init();
