<?php
/**
 * Smart Wishlist Analytics
 *
 * @author MoreConvert
 * @package Smart Wishlist For More Convert
 * @since 1.4.1
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'WLFMC_Analytics' ) ) {
	/**
	 * WooCommerce Wishlist Analytics
	 */
	class WLFMC_Analytics {


		/**
		 * Single instance of the class
		 *
		 * @var WLFMC_Analytics
		 */
		protected static $instance;


		/**
		 * Constructor
		 *
		 * @return void
		 */
		public function __construct() {
			$this->hooks();
		}

		/**
		 * Define Hooks
		 *
		 * @return void
		 */
		public function hooks() {

			add_action( 'wlfmc_product_added_to_cart', array( $this, 'add_item_data' ), 10, 4 );
			add_action( 'woocommerce_checkout_create_order', array( $this, 'add_order_item_meta' ) );

			add_action( 'woocommerce_remove_cart_item', array( __CLASS__, 'remove_item_data' ) );
			add_action( 'woocommerce_cart_emptied', array( __CLASS__, 'remove_item_data_cart_session' ) );

			add_action( 'woocommerce_order_status_changed', array( $this, 'order_status_analytics' ), 9, 3 );

			add_action( 'wlfmc_added_to_wishlist', array( $this, 'added_to_analytics' ), 10, 4 );

		}

		/**
		 * Add product to analytics after added to list.
		 *
		 * @param int                 $product_id Product id.
		 * @param int                 $wishlist_id Wishlist id.
		 * @param int                 $user_id User id.
		 * @param WLFMC_Wishlist_Item $item Wishlist item.
		 *
		 * @return false|int|void
		 */
		public function added_to_analytics( int $product_id, int $wishlist_id, int $user_id, $item ) {
			global $wpdb;

			if ( ! $user_id > 0 ) {
				return false;
			}

			$exists = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->wlfmc_analytics WHERE prod_id = %d AND wishlist_id = %d AND user_id = %d AND  DATE(dateadded) = CURDATE() LIMIT 1", $product_id, $wishlist_id, $user_id ) );// phpcs:ignore WordPress.DB
			if ( ! $exists ) {
				$product = wc_get_product( $product_id );

				if ( ! $product ) {
					return 0;
				}

				$wpdb->insert( // phpcs:ignore WordPress.DB
					$wpdb->wlfmc_analytics,
					array(
						'prod_id'     => $product_id,
						'wishlist_id' => $wishlist_id,
						'user_id'     => $user_id,
						'price'       => $item->get_original_price( 'edit' ),
						'currency'    => get_woocommerce_currency(),
					),
					array(
						'%d',
						'%d',
						'%d',
						'%f',
						'%s',
					)
				);

			}

		}

		/**
		 * Insert product to analytics and set sell with list if coupon used set to coupon.
		 *
		 * @param integer $order_id Order id.
		 * @param integer $user_id User id.
		 * @param integer $wishlist_id Wishlist id.
		 * @param integer $product_id Product id.
		 * @param integer $quantity product Quantity.
		 * @param string  $price Product price.
		 * @param string  $type  buy through list or coupon.
		 *
		 * @return void
		 */
		public function sell_product_with_lists( int $order_id, int $user_id, int $wishlist_id, int $product_id, int $quantity, string $price, string $type = 'buy-through-list' ) {
			global $wpdb;
			$exists_id = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->wlfmc_analytics WHERE prod_id = %d AND wishlist_id = %d AND user_id = %d AND type='add-to-list'  LIMIT 1", $product_id, $wishlist_id, $user_id ) );// phpcs:ignore WordPress.DB
			if ( $exists_id ) {
				$wpdb->update( // phpcs:ignore WordPress.DB
					$wpdb->wlfmc_analytics,
					array(
						'order_id'      => $order_id,
						'quantity'      => $quantity,
						'price'         => $price,
						'type'          => $type,
						'datepurchased' => current_time( 'mysql' ),
						'currency'      => get_woocommerce_currency(),
					),
					array( 'ID' => $exists_id ),
					array(
						'%d',
						'%d',
						'%f',
						'%s',
						'%s',
						'%s',
					),
					array( '%d' )
				);
			} else {
				$wpdb->insert( // phpcs:ignore WordPress.DB
					$wpdb->wlfmc_analytics,
					array(
						'order_id'      => $order_id,
						'user_id'       => $user_id,
						'prod_id'       => $product_id,
						'wishlist_id'   => $wishlist_id,
						'quantity'      => $quantity,
						'price'         => $price,
						'type'          => $type,
						'datepurchased' => current_time( 'mysql' ),
						'currency'      => get_woocommerce_currency(),
					),
					array(
						'%d',
						'%d',
						'%d',
						'%d',
						'%d',
						'%f',
						'%s',
						'%s',
						'%s',
					)
				);
			}

		}

		/**
		 * Insert product to analytics and set sell with coupons.
		 *
		 * @param integer $order_id Order id.
		 * @param integer $user_id User id.
		 * @param integer $prod_id Product id.
		 * @param integer $quantity Quantity.
		 * @param string  $price Product price.
		 *
		 * @return void
		 */
		public function sell_product_with_coupons( $order_id, $user_id, $prod_id, $quantity, $price ) {
			global $wpdb;

			$wpdb->insert( // phpcs:ignore WordPress.DB
				$wpdb->wlfmc_analytics,
				array(
					'order_id'      => $order_id,
					'user_id'       => $user_id,
					'prod_id'       => $prod_id,
					'wishlist_id'   => 0,
					'quantity'      => $quantity,
					'price'         => $price,
					'type'          => 'buy-through-coupon',
					'datepurchased' => current_time( 'mysql' ),
					'currency'      => get_woocommerce_currency(),
				),
				array(
					'%d',
					'%d',
					'%d',
					'%d',
					'%d',
					'%f',
					'%s',
					'%s',
					'%s',
				)
			);

		}

		/**
		 * Remove product added from wishlist
		 *
		 * @param string|null $cart_item_key Cart product key.
		 *
		 * @return boolean
		 */
		public static function remove_item_data( string $cart_item_key = null ): bool {
			$data = (array) WC()->session->get( 'wlfmc_wishlist_data', array() );
			if ( empty( $cart_item_key ) ) {
				WC()->session->set( 'wlfmc_wishlist_data', array() );

				return true;
			}
			if ( ! array_key_exists( $cart_item_key, $data ) ) {
				return false;
			}
			unset( $data[ $cart_item_key ] );
			WC()->session->set( 'wlfmc_wishlist_data', $data );

			return true;
		}

		/**
		 * Clear wishlist cart session.
		 *
		 * @param bool $clear_persistent_cart Should the persistent cart be cleared too. Defaults to true.
		 */
		public static function remove_item_data_cart_session( $clear_persistent_cart = true ) {
			if ( $clear_persistent_cart ) {
				WC()->session->set( 'wlfmc_wishlist_data', array() );

			}
		}

		/**
		 * Add wishlist data to item in cart
		 *
		 * @param integer $wishlist_id Wishlist id.
		 * @param string  $cart_item_key Cart product key.
		 * @param integer $product_id Product id.
		 * @param integer $quantity Product quantity.
		 *
		 * @return void
		 */
		public function add_item_data( int $wishlist_id, string $cart_item_key, int $product_id, int $quantity ) {
			$data                   = (array) WC()->session->get( 'wlfmc_wishlist_data', array() );
			$data[ $cart_item_key ] = array(
				'wishlist_id' => $wishlist_id,
				'product_id'  => $product_id,
			);
			WC()->session->set( 'wlfmc_wishlist_data', $data );
		}


		/**
		 * Add meta data for product when created order
		 *
		 * @param WC_Order $order Order object.
		 */
		public function add_order_item_meta( WC_Order $order ) {

			foreach ( $order->get_items() as $item ) {
				$data = $this->get_item_data( $item->legacy_cart_item_key );
				if ( ! empty( $data ) ) {
					$item->update_meta_data( '_wlfmc_wishlist_cart', $data );
				}
			}
		}


		/**
		 * Get product added from wishlist
		 *
		 * @param string $cart_item_key Cart product key.
		 *
		 * @return array
		 */
		public function get_item_data( $cart_item_key ) {
			$data = (array) WC()->session->get( 'wlfmc_wishlist_data', array() );
			if ( empty( $data[ $cart_item_key ] ) ) {
				$data[ $cart_item_key ] = array();
			}

			return $data[ $cart_item_key ];
		}


		/**
		 * Analytics check completed orders
		 *
		 * @param integer $order_id Order id.
		 * @param string  $old_status Not used.
		 * @param string  $new_status Updated status order.
		 *
		 * @return void
		 */
		public function order_status_analytics( int $order_id, string $old_status, string $new_status ) {
			$new_status = str_replace( 'wc-', '', $new_status );
			$order      = new WC_Order( $order_id );

			if ( in_array(
				$new_status,
				array(
					'processing',
					'completed',
				),
				true
			) && empty( get_post_meta( $order_id, '_wlfmc_analytics_processed', true ) ) ) {

				$items = $order->get_items();
				if ( empty( $items ) || ! is_array( $items ) ) {
					return;
				}

				$user_id = $order->get_customer_id();

				$coupon_used = false;
				$wishlists   = false;

				foreach ( $order->get_coupon_codes() as $coupon_code ) {
					// Get the WC_Coupon object.
					$coupon           = new WC_Coupon( $coupon_code );
					$analytics_coupon = $coupon->get_meta( 'wlfmc_analytics' );

					if ( in_array( $analytics_coupon, array( 'automation', 'campaign' ), true ) ) {
						$coupon_used = true;
					}
				}
				if ( ! $coupon_used ) {
					$wishlists = WLFMC_Wishlist_Factory::get_wishlists( array( 'user_id' => $user_id ) );
				}
				foreach ( $items as $item ) {

					$data           = array(
						'order_id'    => $order_id,
						'user_id'     => $user_id,
						'quantity'    => $item->get_quantity(),
						'price'       => $item->get_subtotal(),
						'product_id'  => 0,
						'wishlist_id' => 0,
					);
					$find_in_lists  = false;
					$_wishlist_cart = $item->get_meta( '_wlfmc_wishlist_cart' );

					if ( $_wishlist_cart ) {

						$find_in_lists       = true;
						$data['wishlist_id'] = $_wishlist_cart['wishlist_id'];
						$data['product_id']  = $_wishlist_cart['product_id'];

					} else {

						if ( ! empty( $wishlists ) ) {
							foreach ( $wishlists as $wishlist ) {

								$product_id   = $item->get_product_id();
								$variation_id = $item->get_variation_id();

								if ( $variation_id && $wishlist->has_product( $variation_id ) ) {
									$find_in_lists       = true;
									$data['wishlist_id'] = $wishlist->get_id();
									$data['product_id']  = $variation_id;
									break;
								} elseif ( $product_id && $wishlist->has_product( $product_id ) ) {

									$find_in_lists       = true;
									$data['wishlist_id'] = $wishlist->get_id();
									$data['product_id']  = $product_id;
									break;
								}
							}
						}
					}

					if ( $coupon_used && $find_in_lists ) {

						$this->sell_product_with_lists( $data['order_id'], $data['user_id'], $data['wishlist_id'], $data['product_id'], $data['quantity'], $data['price'], 'buy-through-coupon' );

					} elseif ( $coupon_used ) {

						$product_id   = $item->get_product_id();
						$variation_id = $item->get_variation_id();
						$prod_id      = $variation_id ? $variation_id : $product_id;
						$this->sell_product_with_coupons( $order_id, $user_id, $prod_id, $item->get_quantity(), $item->get_subtotal() );

					} elseif ( $find_in_lists ) {

						$this->sell_product_with_lists( $data['order_id'], $data['user_id'], $data['wishlist_id'], $data['product_id'], $data['quantity'], $data['price'] );
					}
				}

				update_post_meta( $order_id, '_wlfmc_analytics_processed', '1' );
			}
		}


		/**
		 * Returns single instance of the class.
		 *
		 * @access public
		 *
		 * @return WLFMC_Analytics
		 */
		public static function get_instance() {
			if ( is_null( self::$instance ) ) {
				self::$instance = new self();
			}

			return self::$instance;
		}


	}

}
/**
 * Unique access to instance of WLFMC_Analytics class.
 *
 * @return WLFMC_Analytics
 */
function WLFMC_Analytics(): WLFMC_Analytics {  // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.FunctionNameInvalid
	return WLFMC_Analytics::get_instance();
}
